#!perl -T
use strict;
use warnings;
use Test::More;

# Ensure a recent version of Test::Pod
my $min_tp = 1.22;
plan skip_all => "Test::Pod $min_tp required for testing POD"
    unless eval "use Test::Pod $min_tp; 1";

all_pod_files_ok();
